import java.io.IOException;
import java.io.Serializable;
import java.net.ServerSocket;
import java.net.Socket;

public class DistMult implements MatrixMult {

    int port = 10000;
    int anzahl; 
    ObjectChannel[] oc;

    public DistMult(int port, int anzahl) throws IOException {
        this.port = port;
        this.anzahl = anzahl;
        ServerSocket ss = new ServerSocket(port);
        System.out.println("DistMult server startet on " + ss);
        oc = new ObjectChannel[anzahl];
        for ( int i = 0; i < anzahl; i++ ) {
            Socket s = ss.accept();
            oc[i] = new ObjectChannel(s);
        }
        System.out.println("DistMult server initializion complete");
    }

    public DistMult(int anzahl) throws IOException {
        this(10000,anzahl);
    }

    public void shutdown() {
        Feierabend fa = new Feierabend(); 
        for ( int i = 0; i < anzahl; i++ ) {
            try {
                oc[i].send(fa);
            } catch (IOException e) {
            }
            oc[i].close();
            //oc[i].s.close();
        }
        //ss.close();
    }
        
    /**
     * Performs the multiplication of two matrices.
     * C = A * B.
     * @param C result matrix.
     * @param A matrix.
     * @param B matrix.
     */
    public void multiply(double[][] C, double[][] A, double[][] B) {
        System.out.print("Starting " + anzahl + " remote processes ...");
        for (int i=0; i < anzahl; i++) {
            RowMultDist rmd = new RowMultDist(C,A,B,i,anzahl);
            try {
                oc[i].send(rmd);
            } catch (IOException e) {
                throw new RuntimeException("" + e);
            }
        }
        System.out.print(" send work ...");
        int k = 0;
        for (int i=0; i < anzahl; i++) {
            try { 
                Object o = oc[i].receive(); 
                if ( o instanceof RowMultDist ) {
                    RowMultDist rmd = (RowMultDist) o;
                    // copy back rmd.C to C
                    for (int j = 0; j < rmd.C.length; j++) {
                        //A[k] = rmd.A[j];
                        C[k] = rmd.C[j];
                        k++;
                    }
                }
            } catch (ClassNotFoundException e) {
                throw new RuntimeException("" + e);
            } catch (IOException e) {
                throw new RuntimeException("" + e);
            }
        }
        System.out.println(" done DistMult");
    }
}
        
/**
 * It performs a row multiplication.
 */
class RowMultDist implements Serializable, Runnable {
    double[][] A;
    double[][] B;
    double[][] C;
    int i;  
    int anzahl;
    int zeilen;

    /**
     * Constructor.
     * @param Cp Two dimensional Matrices.
     * @param Ap Two dimensional Matrices.
     * @param Bp Two dimensional Matrices.
     */
    RowMultDist(double[][] Cp, double[][] Ap, double[][] Bp, 
                int ip, int az) {
        B = Bp; i = ip; anzahl = az;
        zeilen = Ap.length / anzahl; 
        if ( Ap.length % anzahl != 0 ) {
            zeilen++;
        }
        A = new double[zeilen][];
        C = new double[zeilen][];
        int end = (i+1)*zeilen;
        if ( end > Ap.length ) {
            end = Ap.length;
        }
        int k = 0;
        for ( int j = i*zeilen; j < end; j++ ) {
            A[k] = Ap[j];
            C[k] = Cp[j];
            k++;
        }
    }

    /**
     * Runs the multiplication.
     */
    public void run() {
        for ( int i = 0; i < A.length; i++ ) {
            for (int j=0; j < B[0].length; j++) {
                double c = 0.0;
                for (int k=0; k < B.length; k++) {
                    c += A[i][k] * B[k][j];
                }
                C[i][j] = c;
            }
        }
        B = null; // damit Rücktransport gespart wird
        A = null;
    }
}
