public class ParMultCoreBlockTrans implements MatrixMult {
    private int threads;
    private int bsize;

    /**
     * Constructor with number of threads.
     * @param threads
     */
    public ParMultCoreBlockTrans(int threads, int bsize) {
        this.threads = threads;
        this.bsize = bsize;
    }

    /**
     * Performs the multiplication of two matrices.
     * C = A * B.
     * @param C result matrix.
     * @param A matrix.
     * @param B matrix.
     */
    public void multiply(double[][] C, double[][] A, double[][] B) {
        Thread[] t = new Thread[threads];
        System.out.print("Starting ParMultCoreBlockTrans " 
                         + threads + " threads, block size "
                         + bsize);
        for (int i=0; i < threads; i++) {
            t[i] = new RowMultCoreBlockTrans(C,A,B,i,bsize,threads);
            t[i].start();
        }
        System.out.print(" started ...");
        for (int i=0; i < threads; i++) {
            try { t[i].join(); }
            catch (InterruptedException e) { }
        }
        System.out.println(" done");
    }
}


/**
 * This class is derived from the class Thread.
 * It performs a row multiplication.
 */
class RowMultCoreBlockTrans extends Thread {
    final double[][] A;
    final double[][] B;
    final double[][] C;
    final int i;  
    final int bsize;
    final int threads;

    /**
     * Constructor.
     * @param Cp Two dimensional result matrix.
     * @param Ap Two dimensional input matrix.
     * @param Bp Two dimensional input matrix.
     */
    RowMultCoreBlockTrans(double[][] Cp, 
                          double[][] Ap, double[][] Bp, 
                          int ip, int bsize, int threads) {
        A = Ap; B = Bp; C = Cp; i = ip; 
        this.bsize = bsize;
        this.threads = threads;
    }

    /**
     * Runs the multiplication.
     */
    public void run() {
        int zeilen = A.length / threads;
        if ( A.length % threads != 0 ) {
            zeilen++; // wenns nicht aufgeht
        }
        int end = (i+1)*zeilen;
        if (end > A.length) {
            end = A.length; // Begrenzung auf Länge von A
        }
        for (int ii = i*zeilen; ii < end; ii += bsize) {
            int endi = ii + bsize;
            if ( endi > A.length ) {
                endi = A.length;
            }
            for ( int jj = 0; jj < B.length; jj += bsize) {
                int endj = jj + bsize;
                if ( endj > B.length ) {
                    endj = B.length;
                }
                for (int i=ii; i < endi; i++) {
                    for (int j=jj; j < endj; j++) {
                        double c = 0.0;
                        for (int k=0; k < B[j].length; k++) {
                            c += A[i][k] * B[j][k];// tausch
                        }
                        C[i][j] = c;
                    }
                }
            }
        }
    }			
}
