
package algo;

import java.io.Serializable;
import java.util.Random;

/**
 * A Point in a 2d Graph.
 * Defined by an x- and y-coordinate and given a number.
 * @author Heinz Kredel. 
 */
public class Point implements Serializable {

    public final int n;
    public final double x;
    public final double y;

/**
 * @param n the number of the point
 * @param x coordinate.
 * @param y coordinate.
 */
    public Point(int n, double x, double y) {
        this.n = n;
        this.x = x;
        this.y = y;
    }


    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return "["+n+"]("+x+","+y+")";
    }


/**
 * @param n the number of the point
 * @param r a random number generator.
 * @param m maximal coordinate range.
 * @return point with random x- and y-coordinates less than m.
 */
    public static Point random(int n, Random r, double m) {
        return new Point(n,
                         Math.abs(r.nextDouble()*m),
                         Math.abs(r.nextDouble()*m));
    }

/**
 * @param p a Point to be scaled.
 * @param scale a scale factor.
 * @return a point with x- and y-coordinates multiplied by scale.
 */
    public static Point scaleTo(Point p, double scale) {
        return new Point(p.n, p.x*scale, p.y*scale);
    }

}