#include "ParMult.h"

#include "unistd.h"

#include <iostream>

using namespace std;


/**
 * Parallel Matrix Multiplication. 
 * Using dim(A) number of threads.
 * @author Joachim Staats.
 */

/**
 * Performs the multiplication of two matrices.
 * C = A * B.
 * @param C result matrix.
 * @param A matrix.
 * @param B matrix.
 */
void ParMult::multiply(double* C, double* A, double* B, int m, int p, int n) 
{
    pthread_t t[m];
    seven_args params[m];
    int thread_present[m];
    int res;
    cout << "Starting " << m << " threads ...";
    for (int i=0; i < m; i++) {
        thread_present[i] = 1;
        params[i].C = C;
        params[i].A = A;
        params[i].B = B;
        params[i].m = m;
        params[i].p = p;
        params[i].n = n;
        params[i].i = i;        
        res = pthread_create(&t[i], NULL, RowMult,(void *)&params[i]);
        if (res != 0)
        {
            thread_present[i] = 0;
            RowMult(&params[i]);
        }
    }
    cout << " started ...";
    for (int i=0; i < m; i++) {
        if (thread_present[i] != 0)
            pthread_join (t[i], NULL);
    }
    cout << " done ParMult ";
}


/**
 * This class is derived from the class Thread.
 * It performs a row multiplication.
 */
void* RowMult (void *arg)
{
    seven_args *params = (seven_args *)arg;
    double* C = params->C;
    double* A = params->A;
    double* B = params->B;
    int m = params->m;
    int p = params->p;
    int n = params->n;
    int i = params->i;

    for (int j=0; j < n; j++) {
      double c = 0.0;
      for (int k=0; k < p; k++) {
          c += A[i*p+k] * B[k+j*p];
      }
      C[i*n+j] = c;
    }
//    cout << C[i*n+n-1];
}
