#include "ParMultProcBlock.h"
#include "ParMultProcBlockTrans.h"

#include "unistd.h"

#include <iostream>
#include <algorithm>

using namespace std;

/**
 * Parallel Matrix Multiplication. 
 * Using specified number of threads, 
 * generating product matrix in blocks, matrix B is transposed.
 * @author Joachim Staats.
 */

/**
 * Performs the multiplication of two matrices, B transposed.
 * C = A * transpose(B).
 * @param C result matrix.
 * @param A matrix.
 * @param B matrix.
 */
void ParMultProcBlockTrans::multiply(double* C, double* A, double* B, int m, int p, int n)
{
    pthread_t t[anzahl];
    nine_args params[m];
    int thread_present[m];
    int res;
    cout << "Starting " << anzahl << " threads";
    cout << ", bs = " << (m/anzahl) << " ...";
    cout << ", na = " << blocksize << " ...";
    for (int i=0; i < anzahl; i++) {
        thread_present[i] = 1;
        params[i].C = C;
        params[i].A = A;
        params[i].B = B;
        params[i].m = m;
        params[i].p = p;
        params[i].n = n;
        params[i].i = i; 
        params[i].anzahl = anzahl;
        params[i].blocksize = blocksize;
        res = pthread_create(&t[i], NULL, RowMultProcBlockTrans,(void *)&params[i]);
        if (res != 0)
        {
            thread_present[i] = 0;
            RowMultProcBlockTrans(&params[i]);
        }
    }
    cout << " started ...";
    for (int i=0; i < anzahl; i++) {
        if (thread_present[i] != 0)
            pthread_join (t[i], NULL);
    }
    cout << " done ParMultProcBlockTrans ";
}


/**
 * This function performs a row multiplication.
 */
void* RowMultProcBlockTrans (void *arg) 
{
    nine_args *params = (nine_args *)arg;
    double* C = params->C;
    double* A = params->A;
    double* B = params->B;
    int m = params->m;
    int p = params->p;
    int n = params->n;
    int i = params->i;
    int anzahl = params->anzahl;
    int blocksize = params->blocksize;

    int schritte = m / anzahl; 
    if ( (m % anzahl) != 0 ) {
        schritte++; // ceiling
    }
    int na = blocksize;
    int nb = blocksize;

    for (int ii=i*schritte; ii < min((i+1)*schritte,m); ii+=na) {
        for (int jj=0; jj < p; jj+=nb) {

            for (int iii = ii; iii < min((ii+na),m); iii++ ) {
                double* Ai = &A[iii];
                for (int j=jj; j < min((jj+nb),p); j++) {
                    double* Bj = &B[j];
                    double c = 0.0;
                    for (int k=0; k < n; k++) {
                        c += Ai[k] * Bj[k];
                    }
                    C[iii*p+j] = c;
                }
            }

        }
    }
}
