
import java.util.concurrent.Executors;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.ExecutionException;


/**
 * Parallel Matrix Multiplication using util.concurrent. 
 * Using dim(A) number of parallel tasks but given number of threads.
 */

public class ConMult implements MMInf {

    int anzahl = 1;

    public ConMult(int threads) {
         anzahl = threads;	
    }


/**
 * Performs the multiplication of two matrices.
 * C = A * B.
 * @param C result matrix.
 * @param A matrix.
 * @param B matrix.
 */
    public void multiply(double[][] C, double[][] A, double[][] B) {
      ExecutorService pool = Executors.newFixedThreadPool(anzahl);
      Future[] f = new Future[A.length];
      System.out.print("Starting " + A.length + " tasks ...");
      for (int i=0; i < A.length; i++) {
          f[i] = pool.submit( new ConRowMult(C,A,B,i) );
      }
      System.out.print(" started ...");
      for (int i=0; i < A.length; i++) {
          try { 
              f[i].get(); 
          } catch (InterruptedException ignored) { 
          } catch (ExecutionException ignored) { 
          }
      }
      pool.shutdown();
      System.out.println(" done ConMult");
    }


    public void multiplyDoesNotWork(double[][] C, double[][] A, double[][] B) {
      ExecutorService pool = Executors.newFixedThreadPool(anzahl);
      System.out.print("Starting " + A.length + " tasks ...");
      for (int i=0; i < A.length; i++) {
          pool.submit( new ConRowMult(C,A,B,i) );
      }
      System.out.print(" started ...");
      pool.shutdown();
      System.out.println(" done ConMult");
    }

}


/**
 * This class is derived from the class Thread.
 * It performs a row multiplication.
 */

class ConRowMult extends Thread {

    double[][] A;
    double[][] B;
    double[][] C;
    int i;  

/**
 * Constructor.
 * @param Cp two dimensional result matrix.
 * @param Ap two dimensional matrix.
 * @param Bp two dimensional matrix.
 */
  ConRowMult(double[][] Cp, double[][] Ap, double[][] Bp, int ip) {
    A = Ap; B = Bp; C = Cp; i = ip;
  }

/**
 * Runs the multiplication.
 */
  public void run() {
    for (int j=0; j < B[0].length; j++) {
      double c = 0.0;
      for (int k=0; k < B.length; k++) {
          c += A[i][k] * B[k][j];
      }
      C[i][j] = c;
    }
  }
}

