
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.concurrent.locks.Condition;

/**
 * Konto version C.
 * Uses java.util.concurrent mutexes to synchronize and coordinate. 
 * Implements methods to display, add and remove money from the account, 
 * account cannot become negative.
 * @author Heinz Kredel.
 */

public class KontoC implements Konto {

    protected double stand; // Konto Stand

    protected final Lock      mutex  = new ReentrantLock();
    protected final Condition imPlus = mutex.newCondition();

    public KontoC() { 
        this(0.0); 
    }

    public KontoC(double a) { 
        stand = a; 
    }


/**
 * Displays account balance.
 * @return stand.
 */
    public double zeigeGeld() { 
        mutex.lock();
        try {
            return stand; 
        } finally {
            mutex.unlock();
        }
    }


/**
 * Transfers amount to this account.
 * Blocks if account would become negative.
 * @param b amount to put on account.
 * @return s returns actual balance from account.
 */
    public double putGeld(double b) { // stand += b
        mutex.lock();
        try {
            double s = stand + b;
            WorkC.schaffen(100);
            // nur falls s >= 0
            while (s < 0 && stand > s) { // increasing negative allowed
                  try { 
                      imPlus.await();
                  } catch(InterruptedException e) { 
                  } 
                  s = stand + b;
            }
            if (s >= 0 || s > stand) imPlus.signal();
            stand = s;
            return s;   
        } finally {
            mutex.unlock();
        }
    }


/**
 * New account balance gets calculated and returned.
 * Blocks if account would become negative.
 * @param b amount to remove from desired account.
 * @return s current account balance.
 */
    public double getGeld(double b) { // stand -= b
        mutex.lock();
        try {
            double s = stand - b;
            WorkC.schaffen(100);
            // nur falls s >= 0
            while (s < 0 && stand > s) { // increasing negative allowed
                  try { 
                      imPlus.await();       
                  } catch(InterruptedException e) { 
                  } 
                  s = stand - b;
            }
            if (s >= 0 || s > stand) imPlus.signal();
            stand = s;
            return s;   
        } finally {
            mutex.unlock();
        }
    }

}


/**
 * Sending current thread to sleep for a random time.
 */
class WorkC {

/**
 * @param max An Integer, defines range of numbers for Math.random().
 */
    public static void schaffen(int max) {
        try {
            Thread.currentThread().sleep( (int)(max*Math.random()) );
        } catch (InterruptedException e) {
              System.out.println("Dies sollte nicht passieren "+e);
        }
    }
}
