

import java.io.IOException;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;


/**
 * Versuch mit Object Streams und nio.Buffer
 */

public class ObjectBufferTest {

    final static int SIZE = 4*1024; 

    public static void main( String[] args ) {
        testOutput();
        testInput();
    }


    /**
     * Output 3 objects to a file using a buffer.
     */
    private static void testOutput() {
        ByteBuffer bbd = ByteBuffer.allocate( SIZE );
        bbd.clear();
        ObjectBuffer ob = new ObjectBuffer( bbd );
        System.out.println("ob = " + ob );

        try {
            Object tmp = new java.util.HashMap(44);
            System.out.println("tmp = " + tmp);
            ob.put( tmp );

            tmp = new java.util.Date();
            System.out.println("tmp = " + tmp);
            ob.put( tmp );

            tmp = new java.math.BigInteger("393993");
            System.out.println("tmp = " + tmp);
            ob.put( tmp );
        } catch(IOException e) {
            e.printStackTrace();
            return;
        }
        System.out.println("ob = " + ob );
        bbd.flip();

        String filename = "File2Name";
        FileOutputStream fos = null;
        try {
            fos = new FileOutputStream( filename /*, true*/ );
        } catch(IOException e) {
            e.printStackTrace();
            return;
        }
        FileChannel fc = fos.getChannel();
        try {
            fc.write( bbd );
            fc.force(true);
            fc.close();
        } catch(IOException e) {
            e.printStackTrace();
            return;
        }
        //System.out.println("bbd = " + bbd );
        bbd.rewind();
        //System.out.println("bbd.size = " + bbd.limit() );
        System.out.println("+++------------------");
    }



    /**
     * Input 3 objects from a file using a buffer.
     */
    private static void testInput( ) {
        String filename = "File2Name";
        FileInputStream fis = null;
        try {
            fis = new FileInputStream( filename );
        } catch(IOException e) {
            e.printStackTrace();
            return;
        }
        FileChannel fc = fis.getChannel();
        ByteBuffer bbw = ByteBuffer.allocateDirect( SIZE );
        bbw.clear();
        int count = 0;
        try {
            count = fc.read( bbw ); 
            //System.out.println("count = " + count);
            if ( count >= SIZE ) { // should not happen
                System.out.println("need a loop");
            }
            fc.close();
        } catch(IOException e) {
            e.printStackTrace();
            return;
        }
        bbw.flip();

        ObjectBuffer ob = new ObjectBuffer( bbw );
        System.out.println("ob = " + ob );
        Object tmp = null;
        try {
            tmp = ob.get();
            System.out.println("tmp = " + tmp);

            tmp = ob.get();
            System.out.println("tmp = " + tmp);

            tmp = ob.get();
            System.out.println("tmp = " + tmp);
        } catch(IOException e) {
            e.printStackTrace();
            return;
        } catch(ClassNotFoundException e) {
            e.printStackTrace();
            return;
        }
        System.out.println("ob = " + ob );

        System.out.println("***------------------");
    }

}