/**
 * Parallel Matrix Multiplication. 
 * Using specified number of threads, generating product matrix in blocks.
 * @author Heinz Kredel.
 */

public class ParMultProcBlock implements MMInf {

    int anzahl = 1;
    int blocksize = 1;

    public ParMultProcBlock(int threads, int size) {
         anzahl = threads;
         blocksize = size;
    }


/**
 * Performs the multiplication of two matrices.
 * C = A * B.
 * @param C result matrix.
 * @param A matrix.
 * @param B matrix.
 */
    public void multiply(double[][] C, double[][] A, double[][] B) {
      Thread[] t = new Thread[anzahl];
      System.out.print("Starting " + anzahl + " threads");
      System.out.print(", bs = " + (A.length/anzahl) + " ...");
      System.out.print(", na = " + blocksize + " ...");
      for (int i=0; i < anzahl; i++) {
          t[i] = new RowMultProcBlock(C,A,B,i,anzahl,blocksize);
          t[i].start();
      }
      System.out.print(" started ...");
      for (int i=0; i < anzahl; i++) {
          try { t[i].join(); }
          catch (InterruptedException e) { }
      }
      System.out.println(" done ParMultProcBlock");
    }

}


/**
 * This class is derived from the class Thread.
 * It performs a row multiplication.
 */
class RowMultProcBlock extends Thread {

    double[][] A;
    double[][] B;
    double[][] C;
    int i;  
    int anzahl;
    int blocksize;

/**
 * Constructor.
 * @param Cp Two dimensional matrices.
 * @param Ap Two dimensional matrices.
 * @param Bp Two dimensional matrices.
 */
  RowMultProcBlock(double[][] Cp, double[][] Ap, double[][] Bp, 
                   int ip, int a, int s) {
    A = Ap; B = Bp; C = Cp; 
    i = ip; 
    anzahl = a;
    blocksize = s;
  }

/**
 * Runs the multiplication.
 */
  public void run() {
    int schritte = A.length / anzahl; 
    if ( (A.length % anzahl) != 0 ) {
        schritte++; // ceiling
    }
    int na = blocksize;
    int nb = blocksize;

    for (int ii=i*schritte; ii < Math.min((i+1)*schritte,A.length); ii+=na) {
        for (int jj=0; jj < B[0].length; jj+=nb) {

            for (int iii = ii; iii < Math.min((ii+na),A.length); iii++ ) {
                double[] Ai = A[iii];
                for (int j=jj; j < Math.min((jj+nb),B[0].length); j++) {
                    double c = 0.0;
                    for (int k=0; k < B.length; k++) {
                        c += Ai[k] * B[k][j];
                    }
                    C[iii][j] = c;
                }
            }

        }
    }

  }

}
