
/**
 * Parallel Vector Scalar Multiplication.
 * @author Heinz Kredel.
 */

public class ParVSMult implements VSMInf {

    int anzahl = 1;

    public ParVSMult(int threads) {
         anzahl = threads;	
    }



/**
 * Compute the scalar multiplication of two vectors.
 * @param A vector.
 * @param B vector.
 */
    public double multiply(double[] A, double[] B) {
      Thread[] t = new Thread[anzahl];
      System.out.print("Starting " + anzahl + " threads ...");
      DoubleStore c = new DoubleStore(0.0);
      for (int i=0; i < anzahl; i++) {
          t[i] = new VSpMult(c,A,B,i,anzahl);
          t[i].start();
      }
      System.out.print(" ... started ...");
      for (int i=0; i < anzahl; i++) {
          try { t[i].join(); }
          catch (InterruptedException e) { }
      }
      System.out.println(" ... done");
      return c.get();
    }

/**
 * Compute the norm of a vector.
 * @param A vector.
 */
    public double norm(double[] A) {
      Thread[] t = new Thread[anzahl];
      System.out.print("Starting " + anzahl + " threads ...");
      DoubleStore c = new DoubleStore(0.0);
      for (int i=0; i < anzahl; i++) {
          t[i] = new VSpNorm(c,A,i,anzahl);
          t[i].start();
      }
      System.out.print(" ... started ...");
      for (int i=0; i < anzahl; i++) {
          try { t[i].join(); }
          catch (InterruptedException e) { }
      }
      System.out.println(" ... done");
      return Math.sqrt( c.get() );
    }

}


/**
 * Synchronized storage of a double.
 */
class DoubleStore {

    protected double d = 0.0;

    public DoubleStore(double d) {
        this.d = d;
    }

    public synchronized double get() {
        return d;
    }

    public synchronized void add(double c) {
        d += c;
    }

}


/**
 * This class is derived from the class Thread.
 * It performs a scalar multiplication.
 */
class VSpMult extends Thread {

    double[] A;
    double[] B;
    DoubleStore c;
    int i;  
    int anzahl;

/**
 * Constructor.
 * @param Ap one dimensional matrix.
 * @param Bp one dimensional matrix.
 */
  VSpMult(DoubleStore cp, double[] Ap, double[] Bp, int ip, int a) {
    A = Ap; B = Bp; c = cp; i = ip; anzahl = a;
  }

/**
 * Runs the multiplication.
 */
  public void run() {
    int schritte = A.length / anzahl; 
    if ( (A.length % anzahl) != 0 ) {
        schritte++; // ceiling
    }
    double s = 0.0;
    for (int ii = i*schritte; ii < Math.min((i+1)*schritte,Math.min(A.length,B.length)); ii++ ) {
        s += A[ii] * B[ii];
    }
    c.add(s);
  }

}


/**
 * This class is derived from the class Thread.
 * It performs a scalar multiplication of a part.
 */
class VSpNorm extends Thread {

    double[] A;
    DoubleStore c;
    int i;  
    int anzahl;

/**
 * Constructor.
 * @param Ap One dimensional matrix.
 */
  VSpNorm(DoubleStore cp, double[] Ap, int ip, int a) {
    A = Ap; c = cp; i = ip; anzahl = a;
  }

/**
 * Runs the multiplication.
 */
  public void run() {
    int schritte = A.length / anzahl; 
    if ( (A.length % anzahl) != 0 ) {
        schritte++; // ceiling
    }
    double s = 0.0;
    for (int ii = i*schritte; ii < Math.min((i+1)*schritte,A.length); ii++ ) {
        s += A[ii] * A[ii];
    }
    c.add(s);
  }

}
