//package edu.unima.ky.parallel;

/** 
 * Semaphore.
 * This class implements a semaphore.
 * @author Akitoshi Yoshida
 * @author Heinz Kredel.
 */ 

public class Sema {
  private int init;
  private int s;
  private int del;

  /**
   * Constructs a default semaphore.
   */
  public Sema() {
    this(0);
  }

  /**
   * Constructs a semaphore with the given upper limit value.
   * @param i limit.
   */
  public Sema(int i) {
    if (i >=0) { init = i; } else { init = 0; }
    s = init;
    del = 0;
  }

  /**
   * Releases the ressources of the constructed semaphore.
   */
  protected void finalize() throws Throwable {
    if (init != s) { 
        int x = s - init;
        System.out.println("sema: " + x + " pending operations."); 
    }
   super.finalize();
  }

  /**
   * Performs the P operation, which causes the current thread
   * to wait until the semaphore is positive.
   */
  public synchronized void P() {
    while (s <= 0) {
      del++;
      try {
	this.wait(); 
      } catch (InterruptedException e) {}
      del--;
    }
    s--;
  }

  /**
   * The V operation increases the value of s.
   */
  public synchronized void V() {
    s++;
    if (del > 0) {
      this.notify();
    }
  }

  /**
   * checks if Semaphore is positive.
   */
  public boolean isPositive() {
      return (s > 0);
  }

}
