import java.io.IOException;

public class MultiMain {

    /**
     * Generates a matrix.
     * Creates a two dimensional array <CODE>A</CODE> with the dimension n, m.
     * The method <CODE>random</CODE> returns a 
     * random number within the intervall [0,1].
     * @param n dimension of matrix.
     * @param m dimension of matrix.
     */
    public double[][] matgen(int n, int m) {
        double A[][] = new double[n][m];
        for (int i=0; i < n; i++) {
            for (int j=0; j < m; j++) {
                A[i][j] = Math.random();
            }
        }
        return A;
    }


    /** 
     * Generates the null-matrix.
     * @param n dimension of matrix.
     * @param m dimension of matrix.
     */
    public double[][] matgen0(int n, int m) {
        double A[][] = new double[n][m];
        for (int i=0; i < n; i++) {
            for (int j=0; j < m; j++) {
                A[i][j] = 0.0;
            }
        }
        return A;
    }


    /** 
     * Generates the unit-matrix.
     * @param n dimension of matrix.
     * @param m dimension of matrix.
     */
    public double[][] matgen1(int n, int m) {
        double A[][] = new double[n][m];
        for (int i=0; i < n; i++) {
            for (int j=0; j < m; j++) {
                if (i == j) A[i][j] = 1.0; else A[i][j]= 0.0;
            }
        }
        return A;
    }


    /** 
     * Prints out the rows of matrix.
     * @param A Array. 
     */
    public void matprint(double[][] A) {
        for (int i=0; i < A.length; i++) {
            for (int j=0; j < A[0].length; j++) {
                System.out.print(A[i][j] + " ");
            }
            System.out.println();
        }
        System.out.println();
    }
          
    /** 
     * Checks if the matrix is approximatly zero.
     * Checks if the absolut value of the matrix's elements are less than
     * the production of the smallest positive value and 1000.
     * @param A Two dimensional array.
     */ 
    public boolean matcheck0(double[][] A) {
        double eps = Double.MIN_VALUE*1000.0;
        for (int i=0; i < A.length; i++) {
            for (int j=0; j < A[0].length; j++) {
                if ( Math.abs(A[i][j]) > eps) { 
                    System.out.println("(i,j)=("+i+","+j+") = "+ A[i][j]);
                    return false;
                }
            }
        }
        return true;
    }
           
    /** 
     * Performs a subtraction operation between two arrays sequentially.
     * @param C result matrix = A-B.
     * @param A left matrix.
     * @param B right matirx.
     */
    public void diff(double[][] C, double[][] A, double[][] B) {
        for (int i=0; i < C.length; i++) {
            for (int j=0; j < C[0].length; j++) {
                C[i][j] = A[i][j] - B[i][j];
            }
        }
    }
             
    public void transpose(double[][] A) {
        for ( int i = 0 ; i < A.length; i++ ) {
            for ( int j = i+1; j < A[0].length; j++ ) {
                double d = A[j][i];
                A[j][i] = A[i][j];
                A[i][j] = d;
            }
        }
    }
             
    public static void main(String[] args) {
        int n = 800;
        int m = 800;
        MultiMain x = new MultiMain();
        double[][] C = x.matgen0(n,m);
        double[][] A = x.matgen1(n,m);
        double[][] B = x.matgen(n,m);
        double[][] D = x.matgen0(n,m);
        boolean print = false;
        if (print) {
            x.matprint(C);
            x.matprint(A);
            x.matprint(B);
            x.matprint(D);
        }
                
        MatrixMult seq = new SeqMult();
        long ts = System.currentTimeMillis();
        seq.multiply(C,A,B);
        ts = System.currentTimeMillis() - ts;

        //        MatrixMult seqb = new SeqMultBlock(80);
        //        long tsb = System.currentTimeMillis();
        //        seqb.multiply(C,A,B);
        //        tsb = System.currentTimeMillis() - tsb;
        //              
        //        MatrixMult seqbt = new SeqMultBlockTrans(160);
        //        long tsbt = System.currentTimeMillis();
        //        x.transpose(B);
        //        seqbt.multiply(C,A,B);
        //        tsbt = System.currentTimeMillis() - tsbt;
        //        //x.matprint(B);
        //        x.transpose(B);
        //        //x.matprint(B);
        //              
        MatrixMult par = new ParMult();
        long tp = System.currentTimeMillis();
        par.multiply(D,A,B);
        tp = System.currentTimeMillis() - tp;

        //        MatrixMult parc = new ParMultCore(2); // Anzahl threads
        //        long tpc = System.currentTimeMillis();
        //        parc.multiply(D,A,B);
        //        tpc = System.currentTimeMillis() - tpc;
        //              
        //        MatrixMult parcbt = new ParMultCoreBlockTrans(2,80); 
        //        // Anzahl threads, Blocksize / Anzahl threads
        //        long tpcbt = System.currentTimeMillis();
        //        x.transpose(B);
        //        parcbt.multiply(D,A,B);
        //        tpcbt = System.currentTimeMillis() - tpcbt;
        //        x.transpose(B);
        
//         DistMult dm = null;
//         try {
//             dm = new DistMult(2);
//         } catch (IOException e) {
//             e.printStackTrace();
//             return;
//         }
//         D = x.matgen0(n,m);
//         long td = System.currentTimeMillis();
//         dm.multiply(D,A,B);
//         td = System.currentTimeMillis() - td;
//         dm.shutdown();
                  
        if (print) {
            x.matprint(A);
            x.matprint(B);
            x.matprint(C);
            x.matprint(D);
        }
        // C == D ?
        double[][] E = x.matgen0(n,m);
        x.diff(E,C,D);
        boolean t = x.matcheck0(E);
        System.out.println("Matrix: " + n + " x " + m);
        System.out.println("Ergebnis C-D == 0                 " + t);
        System.out.println("Zeit sequential                 = " + ts);
        //        System.out.println("Zeit sequential block           = " + tsb);
        //        System.out.println("Zeit sequential block transpose = " + tsbt);
        System.out.println("Zeit parallel                   = " + tp);
        //        System.out.println("Zeit parallel core              = " + tpc);
        //        System.out.println("Zeit parallel core block trans  = " + tpcbt);
        //System.out.println("Zeit verteilt                   = " + td);
    }
}
