public class ParMult implements MatrixMult {
    /**
     * Performs the multiplication of two matrices.
     * C = A * B.
     * @param C result matrix.
     * @param A matrix.
     * @param B matrix.
     */
    public void multiply(double[][] C, double[][] A, double[][] B) {
        Thread[] t = new Thread[A.length];
        System.out.print("Starting " + A.length + " threads ...");
        for (int i=0; i < A.length; i++) {
            t[i] = new RowMult(C,A,B,i);
            t[i].start();
        }
        System.out.print(" started ...");
        for (int i=0; i < A.length; i++) {
            try { t[i].join(); }
            catch (InterruptedException e) { }
        }
        System.out.println(" done ParMult");
    }
}


/**
 * This class is derived from the class Thread.
 * It performs a row multiplication.
 */
class RowMult extends Thread {
    double[][] A;
    double[][] B;
    double[][] C;
    int i;  

    /**
     * Constructor.
     * @param Cp Two dimensional Matrices.
     * @param Ap Two dimensional Matrices.
     * @param Bp Two dimensional Matrices.
     */
    RowMult(double[][] Cp, double[][] Ap, double[][] Bp, int ip) {
        A = Ap; B = Bp; C = Cp; i = ip;
    }

    /**
     * Runs the multiplication.
     */
    public void run() {
        for (int j=0; j < B[0].length; j++) {
            double c = 0.0;
            for (int k=0; k < B.length; k++) {
                c += A[i][k] * B[k][j];
            }
            C[i][j] = c;
        }
    }
}

