public class ParMultCore implements MatrixMult {
    private int threads;
    /**
     * Constructor with number of threads.
     * @param threads
     */
    public ParMultCore(int threads) {
        this.threads = threads;
    }
    /**
     * Performs the multiplication of two matrices.
     * C = A * B.
     * @param C result matrix.
     * @param A matrix.
     * @param B matrix.
     */
    public void multiply(double[][] C, double[][] A, double[][] B) {
        Thread[] t = new Thread[threads];
        System.out.print("Starting ParMultCore " + threads + " threads ...");
        for (int i=0; i < threads; i++) {
            t[i] = new RowMultCore(C,A,B,i,threads);
            t[i].start();
        }
        System.out.print(" started ...");
        for (int i=0; i < threads; i++) {
            try { t[i].join(); }
            catch (InterruptedException e) { }
        }
        System.out.println(" done");
    }
}


/**
 * This class is derived from the class Thread.
 * It performs a row multiplication.
 */
class RowMultCore extends Thread {
    final double[][] A;
    final double[][] B;
    final double[][] C;
    final int i;  
    final int threads;

    /**
     * Constructor.
     * @param Cp Two dimensional result matrix.
     * @param Ap Two dimensional input matrix.
     * @param Bp Two dimensional input matrix.
     */
    RowMultCore(double[][] Cp, double[][] Ap, double[][] Bp, 
                int ip, int threads) {
        A = Ap; B = Bp; C = Cp; i = ip; 
        this.threads = threads;
    }

    /**
     * Runs the multiplication.
     */
    public void run() {
        int zeilen = A.length / threads;
        if ( A.length % threads != 0 ) {
            zeilen++; // wenns nicht aufgeht
        }
        int end = (i+1)*zeilen;
        if (end > A.length) {
            end = A.length; // Begrenzung auf Länge von A
        }
        //for ( int ii = i; ii < A.length; ii += threads) {
        for( int ii = i*zeilen; ii < end; ii++) {
            for (int j=0; j < B[0].length; j++) {
                double c = 0.0;
                for (int k=0; k < B.length; k++) {
                    c += A[ii][k] * B[k][j];
                }
                C[ii][j] = c;
            }
        }
    }
}
